from jsonfield_compat.util import use_native_jsonfield

if use_native_jsonfield():
    from django.contrib.postgres.fields import JSONField as _JSONFormField
else:
    from jsonfield.forms import JSONFormField as _JSONFormField

JSONFormField = _JSONFormField



import json
from functools import partial

try:
    from django.contrib.postgres.fields import JSONField as NativeJSONField
except ImportError:  # pragma: no cover
    # If using Django 1.8, don't worry about trying to be correct.
    # We won't be using the native field.
    NativeJSONField = object

from django.conf import settings
from django.utils import six
from django.utils.module_loading import import_string
from psycopg2.extras import Json


def _get_jsonfield_encoder_class():
    encoder_class = getattr(settings, 'JSONFIELD_ENCODER_CLASS', None)
    if encoder_class:
        if isinstance(encoder_class, six.string_types):
            encoder_class = import_string(encoder_class)

        return encoder_class


def _get_dumps():
    return partial(json.dumps, cls=_get_jsonfield_encoder_class())


class _JSONField(NativeJSONField):
    """ Patches Django's JSONField to use django-jsonfield's
        JSONFIELD_ENCODER_CLASS setting
    """
    def get_prep_value(self, value):
        if value is not None:
            return Json(value, dumps=_get_dumps())
        return value

    def get_prep_lookup(self, lookup_type, value):
        if lookup_type in ('has_key', 'has_keys', 'has_any_keys'):
            return value
        if isinstance(value, (dict, list)):
            return Json(value, dumps=_get_dumps())
        return super(_JSONField, self).get_prep_lookup(lookup_type, value)

    def deconstruct(self):
        name, path, args, kwargs = super(_JSONField, self).deconstruct()
        kwargs.setdefault('default', dict)
        path = 'jsonfield_compat.JSONField'
        return name, path, args, kwargs



from __future__ import print_function

from django.db import connection
from django.db.models.signals import post_migrate

from jsonfield_compat.fields import JSONField
from jsonfield_compat.util import is_db_postgresql, use_native_jsonfield, \
    django_supports_native_jsonfield


def convert_column_to_json(model, column_name):
    if not is_db_postgresql() or not django_supports_native_jsonfield():
        return

    table_name = model._meta.db_table

    with connection.cursor() as cursor:
        cursor.execute(
            "select data_type from information_schema.columns "
            "where table_name = %s and column_name = %s;",
            [table_name, column_name])

        current_type = cursor.fetchone()[0].upper()
        expected_type = 'JSONB' if use_native_jsonfield() else 'TEXT'

        if current_type != expected_type:
            print("{app}: Converting {col} to use native {type} field".format(
                app=model._meta.app_label, col=column_name, type=expected_type))

            cursor.execute(
                "ALTER TABLE {table} ALTER COLUMN {col} "
                "TYPE {type} USING {col}::{type};".format(
                    table=table_name, col=column_name, type=expected_type
                )
            )


def convert_model_json_fields(model):
    json_fields = [f for f in model._meta.fields if f.__class__ == JSONField]

    for field in json_fields:
        _, column_name = field.get_attname_column()
        convert_column_to_json(model, column_name)


def handler_convert_json_fields(sender, **kwargs):
    for model in list(sender.get_models()):
        convert_model_json_fields(model)


def register_app(app_config):
    post_migrate.connect(handler_convert_json_fields, sender=app_config)



from __future__ import print_function

from django import VERSION as DJANGO_VERSION
from django.conf import settings
from django.db import connection


def is_db_postgresql():
    return connection.vendor == 'postgresql'


def django_supports_native_jsonfield():
    return (DJANGO_VERSION[0] == 1 and DJANGO_VERSION[1] >= 9) or DJANGO_VERSION[0] > 1


def use_native_jsonfield():
    return django_supports_native_jsonfield() and is_db_postgresql() and \
        getattr(settings, 'USE_NATIVE_JSONFIELD', False)



from setuptools import setup

setup(
    name="django-jsonfield-compat",
    version='0.4.3',
    description="Compatability layer between django-jsonfield and Django's native JSONField",
    long_description=open("README.md").read(),
    url="http://github.com/kbussell/django-jsonfield-compat/",
    author="Keith Bussell",
    author_email="kbussell@gmail.com",
    packages=[
        "jsonfield_compat",
    ],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Framework :: Django',
    ],
    zip_safe=False,
)



from jsonfield_compat.util import use_native_jsonfield

if use_native_jsonfield():
    from jsonfield_compat.compat import _JSONField
else:
    from jsonfield.fields import JSONField as _JSONField

JSONField = _JSONField
